
"""
This script determines if the interaction is consistent with the ceiling/floor effect.
"""

import csv
import argparse
import subprocess
import sys
import os

def ceiling_floor_effect():


	##################################################################################################
	parser = argparse.ArgumentParser(description = "This script determines if the interaction is consistent with the ceiling/floor effect.")
	parser.add_argument('-i', '--input_main_effect_file', help = "Specify the path to the main effects file generated by the main_effect_extract.py script. This is required.", 
						action='store', required = True)
	parser.add_argument('-o', '--output_results', help = "Specify the the name of the results file. This is required.", 
						action='store', required = True)
	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.input_main_effect_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################
	
	fin_pairs = open(args.input_main_effect_file, 'rU')
	reader = csv.reader(fin_pairs, delimiter='\t')                
	
	fout_ceiling = open(args.output_results, 'w')
	writer_ceiling = csv.writer(fout_ceiling, delimiter='\t')

	#First, lets check into the signicance of these main effects
	for row in reader: 
		gene        = row[0] 
		rsid_1      = row[1] 
		rsid_2      = row[2]
		b1          = float(row[7])
		lcl1 		= float(row[9])	
		rcl1 		= float(row[10])	
		b1d         = row[11]
		lcl1d 		= row[13]
		rcl1d 		= row[14]
		if b1d == "Not Found":
			b1d = 0
			lcl1d = 0
			rcl1d = 0

		b1d         = float(b1d)
		lcl1d 		= float(lcl1d)
		rcl1d 		= float(rcl1d)
		b2          = float(row[15])
		lcl2 		= float(row[17])
		rcl2 		= float(row[18])
		b2d         = row[19]
		lcl2d 		= row[21]
		rcl2d 		= row[22]
		#Some SNPs have a value of NOT FOUND.
		#I am checking for this, and then setting these cases to zero. 
		#The actual value here is only used in comparison to other betas.
		#So, if it's zero, it'll never be greater than the others.
		if b2d == "Not Found":
			b2d = 0
			lcl2d = 0
			rcl2d = 0
		b2d         = float(b2d)
		lcl2d 		= float(lcl2d)
		rcl2d 		= float(rcl2d)
		
		#calculate the significant ME
		sigb1 = sigb1d = sigb2 = sigb2d = 'Not Calc'
		
		if ((lcl1 == "Not Found") or (rcl1 == "Not Found")):
			sigb1 = "Not Found"
		elif lcl1 <= 0 <= rcl1:
			sigb1 = "Not Sig"
		else:
			sigb1 = "Sig"
		
		if ((lcl1d == "Not Found") or (rcl1d == "Not Found")):
			sigb1d = "Not Found"
		elif lcl1d <= 0 <= rcl1d:
			sigb1d = "Not Sig"
		else:
			sigb1d = "Sig"
		
		if ((lcl2 == "Not Found") or (rcl2 == "Not Found")):
			sigb2 = "Not Found"
		elif lcl2 <= 0 <= rcl2:
			sigb2 = "Not Sig"
		else:
			sigb2 = "Sig"   
		
		if ((lcl2d == "Not Found") or (rcl2d == "Not Found")):
			sigb2d = "Not Found"
		elif lcl2d <= 0 <= rcl2d:
			sigb2d = "Not Sig"
		else:
			sigb2d = "Sig"


		#Here, the goal is to determine how many SNPs have a significant main effect (0,1,2)
		number_me = "NONE"
		
		if sigb1 == "Sig":
		 	if sigb2 == "Sig": 
				number_me = 2
			elif sigb2d == "Sig":
				number_me = 2
		elif sigb1d == "Sig":
		 	if sigb2 == "Sig": 
				number_me = 2
			elif sigb2d == "Sig":
				number_me = 2

		#Now that I know how many significant main effects there are, I need to start figuring out the interaction data for each of these index pairs.
		#Below is the interaction data that is important outside of the upcoming section.
		sigb12 = sigb12d = sigb1d2 = sigb1d2d = "Not Calculated"
		max_interaction_term = "Not Calculated"
		

		b12 = float(row[23])
		se12 = float(row[24])
		b12d = float(row[25])
		se12d = float(row[26])
		b1d2 = float(row[27])
		se1d2 = float(row[28])
		b1d2d = float(row[29])
		se1d2d = float(row[30])
		
		lclb12 = b12 - se12
		rclb12 = b12 + se12
		lclb12d = b12d - se12d
		rclb12d = b12d + se12d
		lclb1d2 = b1d2 - se1d2
		rclb1d2 = b1d2 + se1d2
		lclb1d2d = b1d2d - se1d2d
		rclb1d2d = b1d2d + se1d2d
		
		#list of all the significant interaction terms.
		significant_interaction_terms = []
		
		if lclb12 <= 0 <= rclb12:
			sigb12 = "Not Sig"
		else:
			sigb12 = "Sig"
			significant_interaction_terms.append(b12)
			
		if lclb12d <= 0 <= rclb12d:
			sigb12d = "Not Sig"
		else:
			sigb12d = "Sig"
			significant_interaction_terms.append(b12d)
			
		if lclb1d2 <= 0 <= rclb1d2:
			sigb1d2 = "Not Sig"
		else:
			sigb1d2 = "Sig"   
			significant_interaction_terms.append(b1d2)
			
		if lclb1d2d <= 0 <= rclb1d2d:
			sigb1d2d = "Not Sig"
		else:
			sigb1d2d = "Sig"
			significant_interaction_terms.append(b1d2d)

		#Here, I determine what the maximum interaction term is
		max_pos_beta = max(significant_interaction_terms)  #So, this is the greatest positve term
		max_neg_beta = min(significant_interaction_terms)  #This is the greatest negative term
		#We need to see which has the greater absolute value.  Whichever one does is the most significant interaction term.
		if abs(max_pos_beta) > abs(max_neg_beta):
			max_interaction_term = max_pos_beta
		elif abs(max_pos_beta) < abs(max_neg_beta):
			max_interaction_term = max_neg_beta
		elif max_pos_beta == max_neg_beta:
			max_interaction_term = max_neg_beta
		
#OK. Now I know the number of main effects for each index pair, which interaction terms are significant for that pair, and what the largest interaction term actually is.
#Before continuining, I want to stratify my analysis based on the number of main effects present within each index pair.        
		
					
		largest_effect_SNP1 = 0
		largest_effect_SNP2 = 0
		
		#figure out which of them has the largest effect
		#SNP1
		if ((sigb1 == "Sig") and (sigb1d != "Sig")):
			largest_effect_SNP1 = b1
		elif ((sigb1 != "Sig") and (sigb1d == "Sig")):
			largest_effect_SNP1 = b1d
		elif ((sigb1 == "Sig") and (sigb1d == "Sig")):
			if abs(b1) > abs(b1d):
				largest_effect_SNP1 = b1
			elif abs(b1d) > abs(b1):
				largest_effect_SNP1 = b1d
		
		#for SNP2
		if ((sigb2 == "Sig") and (sigb2d != "Sig")):
			largest_effect_SNP2 = b2
		elif ((sigb2 != "Sig") and (sigb2d == "Sig")):
			largest_effect_SNP2 = b2d
		elif ((sigb2 == "Sig") and (sigb2d == "Sig")):
			if abs(b2) > abs(b2d):
				largest_effect_SNP2 = b2
			elif abs(b2d) > abs(b2):
				largest_effect_SNP2 = b2d

		ceiling = []

		if number_me == 2:
		  
			#if out if they are concordant
			if ((float(largest_effect_SNP1) > 0) and (float(largest_effect_SNP2) > 0)):

				#the main effects are concordant
				#Now we need to figure out if the largest interaction term is concordant
				if float(max_interaction_term) < 0:
					#the interaction term is discordant
					ceiling = [gene, rsid_1,rsid_2, "Ceiling/floor effect"]
					writer_ceiling.writerow(ceiling)

				else: 
					ceiling = [gene, rsid_1, rsid_2, "Clear"]
					writer_ceiling.writerow(ceiling)

					
			elif ((float(largest_effect_SNP1) < 0) and (float(largest_effect_SNP2) < 0)):
				#the main effects are concordant
				if float(max_interaction_term) > 0:
					#the interaction term is discordant
					ceiling = [gene, rsid_1,rsid_2, "Ceiling/floor effect"]
					writer_ceiling.writerow(ceiling)
				else: 
					ceiling = [gene, rsid_1, rsid_2, "Clear"]
					writer_ceiling.writerow(ceiling)
			else:
				ceiling = [gene, rsid_1, rsid_2, "Clear"]
				writer_ceiling.writerow(ceiling)
		else:
			ceiling = [gene, rsid_1, rsid_2, "Clear"]
			writer_ceiling.writerow(ceiling)

	fin_pairs.close()
	fout_ceiling.close()

if __name__ == '__main__':
	ceiling_floor_effect()