"""
This script creates all combinations for the conditional cis-eQTL analysis.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os


##################################################################################################
parser = argparse.ArgumentParser(description = "This script takes a list of significant cis-eQTL for each gene, and creates each combination of significant interactions and nominal cis-eQTL."
	+ "This script creates an output file for each gene in the same directory as the INTERSNP results.")

parser.add_argument('-dir', '--input_ciseqtl_directory', help = "This script assumes that the naming convention from the identify_cis_eqtl.py script (i.e., dirctory_name/GeneID_cis_eqtl_pvaluethreshold.txt)" +
					" This is required", 
					action='store', required = True)
parser.add_argument('-pval', '--pvalue_threshold', help = "Specify the p-value threshold for significance (ex.: 0.05). This is required to know which cis-eQTL to test.", 
					action='store', required = True)
parser.add_argument('-int', '--input_interactions_file', help = "Specify the path to the significant interactions output from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
parser.add_argument('-combi_dir', '--output_combination_directory', help = "Specify the output directory containing the combinations to be tested. Note: The final blackslash should be included (i.e., directory_name/)." +
					" This is required.", action='store', required = True)

#Print error message if no input is provided.
if len(sys.argv)==1:
	parser.print_help()
	sys.exit(1)
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.output_combination_directory+"generate_conditional_combinations.log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.input_interactions_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

############################################################################################


def generate_combinations():

	genes = glob.glob(args.input_ciseqtl_directory+'*cis_eqtl_' + str(args.pvalue_threshold) + '.txt')
	
	print "Generating combinations for " + str(len(genes)) + " genes."
	logfile.write("Generating combinations for " + str(len(genes)) + " genes.\n")

	for gene in genes:
		
		gene = gene.split("/")
		gene = gene[len(gene)-1]
		gene = gene.split("_")[0]

		#print gene

		fin_int = open(args.input_interactions_file)
		int_reader = csv.reader(fin_int, delimiter="\t")

		for interaction in int_reader:
			gene_id = interaction[0]

			if gene_id == gene:

				rsid_1 = interaction[3]
				rsid_2 = interaction[7]

				#combination output file name
				fout_combination_name = args.output_combination_directory + gene + "_" + rsid_1 + "_" + rsid_2 + '_maf05_filtered_pheno_cis_eqtl_' + str(args.pvalue_threshold)+ '_combinations.txt'
				fout_combination = open(fout_combination_name, 'w')
				combination_writer = csv.writer(fout_combination, delimiter='\t')

				cis_eqtl = args.input_ciseqtl_directory + gene + '_maf05_filtered_pheno_cis_eqtl_' + str(args.pvalue_threshold) + '.txt'
				fin_cis = open(cis_eqtl, 'rU')
				cis_reader = csv.reader(fin_cis, delimiter = "\t")
				for cis in cis_reader:
					eqtl = cis[0]
					results = [eqtl, rsid_1, rsid_2]
					combination_writer.writerow(results)
				fin_cis.close()
		fin_int.close()
		fout_combination.close()

if __name__ == '__main__':
	generate_combinations()