"""
This script extracts out significant interactions from the INTERSNP result files.
"""

import csv
import argparse
import subprocess
import sys
import glob



def get_signif():

	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function extracts out all interactions passing a specified p-value threshold, and returns these results in a new file.")

	parser.add_argument('-i', '--input_intersnp_directory', help = "Specify the directory that contains the INTERSNP interaction results. This script assumes that the naming convention is dirctory_name/GeneIDBestMarkerCombi2.txt." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-p', '--pvalue_threshold', help = "Specify the p-value threshold for significance (ex.: 0.05). This is required.", 
						action='store', required = True)
	parser.add_argument('-o', '--output_results', help = "Specify the name of the results file. This is required.", 
						action='store', required = True)
	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = []
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	print "A significance threshold of p < " + str(args.pvalue_threshold) + " is being used."
	logfile.write("A significance threshold of p < " + str(args.pvalue_threshold) + " is being used.\n")

	fout_results = open(args.output_results, 'w')

	interaction_results = args.input_intersnp_directory + "*" + "BestMarkerCombi2.txt"

	gene_count = 0
	interaction_count = 0
	significant_interaction_count = 0

	for interaction_file in glob.glob(interaction_results):
		
		gene_count = gene_count + 1

		gene_id = interaction_file.split("/")
		gene_id = gene_id[len(gene_id)-1]
		gene_id = gene_id[:-20]

		with open(interaction_file, 'rU') as fin_inter:
			fin_inter.readline() #gets rid of header
			lines = filter(None, (line.rstrip() for line in fin_inter)) #gets rid of blank lines
			
			for line in lines:
				
				if (line.startswith(" Note:") or line.startswith(' e.g.:')):
					continue
				
				else:
					
					interaction_count = interaction_count + 1

					pval = line.split("\t")

					if float(pval[9]) < float(args.pvalue_threshold):
						
						results = gene_id + '\t' + line +'\n'
						fout_results.write(results)
						significant_interaction_count = significant_interaction_count + 1

	print "Interactions were found for " + str(gene_count) + " genes."
	logfile.write("Interactions were found for " + str(gene_count) + " genes.\n")
	print "A total of " + str(interaction_count) + " interaction tests were performed."
	logfile.write("A total of " + str(interaction_count) + " interaction tests were performed.\n")
	print "A total of " + str(significant_interaction_count) + " interactions were significant."

	logfile.close()
	fout_results.close()
		

if __name__ == '__main__':
		get_signif()