"""
This script takes the output from a single-marker cis-eQTL analysis and identifies all cis-eQTL at a specified p-value threshold.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os


##################################################################################################
parser = argparse.ArgumentParser(description = "This identifies cis-eQTL from INTERSNP single marker analyses. This script creates an output file for each gene in the same directory as the INTERSNP results..")

parser.add_argument('-i', '--input_intersnp_directory', help = "Specify the directory that contains the INTERSNP interaction results. This script assumes that the naming convention is dirctory_name/GeneIDBestMarkerCombi2.txt." +
					" This is required", 
					action='store', required = True)
parser.add_argument('-p', '--pvalue_threshold', help = "Specify the p-value threshold for significance (ex.: 0.05). This is required.", 
					action='store', required = True)


#Print error message if no input is provided.
if len(sys.argv)==1:
	parser.print_help()
	sys.exit(1)
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.input_intersnp_directory+"identify_cis_eqtl.log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = []
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

############################################################################################

def id_cis_eqtl():

	fdir = args.input_intersnp_directory
	genes = glob.glob(fdir+'*Singlemarker.txt')
	
	print "Identified cis-eQTL for " + str(len(genes))
	logfile.write("Identified cis-eQTL for " + str(len(genes)) + "\n")

	for gene in genes:

		#get INTERSNP output file
		single_marker_fin = open(gene, 'rU')
		header = single_marker_fin.readline()#gets rid of the header
		single_marker_reader = csv.reader(single_marker_fin, delimiter='\t')
		
		#combination output file name
		fout_combination_name = gene[:-16] + 'cis_eqtl_' + str(args.pvalue_threshold) + '.txt'
		fout_combination = open(fout_combination_name, 'w')
		combination_writer = csv.writer(fout_combination, delimiter='\t')
		
		for single_marker in single_marker_reader:
			rsid = single_marker[2]
			pval = single_marker[10]
			#print rsid, pval
			
			if float(pval) < float(args.pvalue_threshold):
				results = [rsid]
				combination_writer.writerow(results)
		fout_combination.close()
		single_marker_fin.close()

if __name__ == '__main__':
	id_cis_eqtl()