
"""
This script extracts the main effects for each interaction from the INTERSNP files. This is necessary to determine if the interaction is consistent with the ceiling/floor effect.
"""

import csv
import argparse
import subprocess
import sys
import os

def extract_main_effects():


	##################################################################################################
	parser = argparse.ArgumentParser(description = "This script extracts the main effects for each interaction from the INTERSNP files. This is necessary to determine if the interaction is consistent with the ceiling/floor effect.")
	parser.add_argument('-i', '--input_interactions_file', help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
	parser.add_argument('-d', '--input_intersnp_directory', help = "Specify the directory that contains the files from the INTERSNP interaction analysis. This script assumes that the naming convention is dirctory_name/GeneID*" +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-o', '--output_results', help = "Specify the the name of the results file. This is required.", 
						action='store', required = True)
	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.input_interactions_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################
	
	sig_res = open(args.input_interactions_file, 'rU')
	reader_sig = csv.reader(sig_res, delimiter='\t')
	
	fout_eval = open(args.output_results, 'w')
	writer_eval = csv.writer(fout_eval, delimiter='\t')
	
	for row_s in reader_sig:
		
		gene = row_s[0]
		test_no = row_s[1]
		rsid_1 = row_s[3]
		rsid_2 = row_s[7]
		b12 = float(row_s[24])
		se12 = float(row_s[25])
		b12d = float(row_s[26])
		se12d = float(row_s[27])
		b1d2 = float(row_s[28])
		se1d2 = float(row_s[29])
		b1d2d = float(row_s[30])
		se1d2d = float(row_s[31])
		
		b0 = se0 = lcl0 = rcl0 = b1 = se1 = lcl1 = rcl1 = b1d = se1d = lcl1d = rcl1d = b2 = se2 = lcl2 = rcl2 =  b2d = se2d = lcl2d = rcl2d = "Not Found"
		additional_stuff = "Not Found"
		
		detailed_info_in_name = args.input_intersnp_directory + gene + 'BestMarkerCombi2Details.txt'

		if os.path.isfile(detailed_info_in_name):

			with open(detailed_info_in_name, 'rU') as detailed_info:
				detailed_info = filter(None, (line.rstrip() for line in detailed_info)) #gets rid of blank lines    
			for index, line in enumerate(detailed_info):
				pattern_to_match = test_no + ': SNP1: ' + rsid_1 + '\t' + 'SNP2: ' + rsid_2
				if line == pattern_to_match:
					print line
					# Extract additional lines from the file
					additional_stuff = detailed_info[index+6:index+11]
					
					# For each line, split around tabs, creating a list of lists
					additional_stuff_lines = [row.split('\t') for row in additional_stuff]
					#print "lines split around tabs:",additional_stuff_lines
					
					#if len(additional_stuff_list) == 4:
					for line in additional_stuff_lines:
						additional_stuff_list = [row.split(':') for row in line]
						#print "lines split around tabs, elements split around colons",additional_stuff_list
						if additional_stuff_list[0][0] == 'b0': 
							b0 = float(additional_stuff_list[0][1])
							se0 = float(additional_stuff_list[1][1])
							lcl0 = float(additional_stuff_list[2][1])
							rcl0 = float(additional_stuff_list[3][1])
						elif additional_stuff_list[0][0] == 'b1': 
							b1 = float(additional_stuff_list[0][1])
							se1 = float(additional_stuff_list[1][1])
							lcl1 = float(additional_stuff_list[2][1])
							rcl1 = float(additional_stuff_list[3][1])
						elif additional_stuff_list[0][0] == 'b1D': 
							b1d = float(additional_stuff_list[0][1])
							se1d = float(additional_stuff_list[1][1])
							lcl1d = float(additional_stuff_list[2][1])
							rcl1d = float(additional_stuff_list[3][1])
						elif additional_stuff_list[0][0] == 'b2': 
							b2 = float(additional_stuff_list[0][1])
							se2 = float(additional_stuff_list[1][1])
							lcl2 = float(additional_stuff_list[2][1])
							rcl2 = float(additional_stuff_list[3][1])
						elif additional_stuff_list[0][0] == 'b2D': 
							b2d = float(additional_stuff_list[0][1])
							se2d = float(additional_stuff_list[1][1])
							lcl2d = float(additional_stuff_list[2][1])
							rcl2d = float(additional_stuff_list[3][1])
		else:
			logfile.write("Cannot find file " + detailed_info_in_name)
			parser.error("Cannot find file " + detailed_info_in_name)

		
		#significance of the main effects
		results=[gene, rsid_1, rsid_2, b0, se0, lcl0, rcl0, b1, se1, lcl1, rcl1, b1d, se1d, lcl1d, rcl1d, b2, se2, lcl2, rcl2, b2d, se2d, lcl2d, rcl2d, b12, se12, b12d, se12d, b1d2, se1d2, b1d2d, se1d2d]
		writer_eval.writerow(results)
	sig_res.close()
	fout_eval.close()

if __name__ == '__main__':
	extract_main_effects()