"""
If the calculate_between_pair_ld.py script was run in parallel, this script combines the output.
"""


import csv
import argparse
import subprocess
import sys
import glob
import os

##################################################################################################
parser = argparse.ArgumentParser(description = "This script just merges together the output from calculate_between_pair_ld.py if it was run in parallel.")

parser.add_argument('-split_name', '--fin_files', help = 'Specify the "output_results" name from calc_between_pair_ld.py script. For example, if the output file was "directory/between_pair_ld.txt", it would have' +
					'and the script was run in parallel, it would have become: "directory/between_pair_ld[1].txt". In this case, please provide directory/between_pair_ld". All files with this format will be merged. This is required.', 
					action='store', required = True)
parser.add_argument('-output_merged', '--output_merged', help = "Specify the name of the file that will contain all the merged results. This is required.", 
					action='store', required = True)
parser.add_argument('-output_clustered', '--output_clustered', help = "Specify the name of the file that will contain all the results, with proper clusterIDs. This is required.", 
					action='store', required = True)

args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.output_clustered[:-4] + ".log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = []
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

######################################################################################################################################

def merge_results():

	#combine all your results into one file
	files_to_merge = glob.glob(args.fin_files+'*.txt')
	with open(args.output_merged, 'w') as outfile:
		for fname in files_to_merge:
			with open(fname) as infile:
				for line in infile:
					outfile.write(line)

	print "There were " + str(len(files_to_merge)) + " files merged together."
	logfile.write("There were " + str(len(files_to_merge)) + " files merged together.\n")

	#open up the merged file to get all the unique clusters
	fin_merge = open(args.output_merged, 'rU')
	merged = csv.reader(fin_merge, delimiter="\t")

	gene_cluster_ids = {}

	for interaction in merged:
		gene_id = interaction[0]
		cluster_id = interaction[32]

		unique_id = str(gene_id) + "_" + str(cluster_id)

		if unique_id not in gene_cluster_ids.keys():
			gene_cluster_ids[unique_id] = []
	fin_merge.close()

	#label all the unique clusters
	for i,j in enumerate(gene_cluster_ids.keys()):
		gene_cluster_ids[j] = i

	print "There are " + str(len(gene_cluster_ids.keys())) + " locus pairs."
	logfile.write("There are " + str(len(gene_cluster_ids.keys())) + " locus pairs.\n")

	#output file
	fout_merged = open(args.output_clustered, 'w')
	merged_writer = csv.writer(fout_merged, delimiter = "\t")

	#output the unique clusters
	fin_merge = open(args.output_merged, 'rU')
	merged = csv.reader(fin_merge, delimiter="\t")

	for interaction in merged:
		gene_id = interaction[0]
		cluster_id = interaction[32]

		unique_id = str(gene_id) + "_" + str(cluster_id)
		interaction[32] = gene_cluster_ids[unique_id] #puts the new cluster ID in

		merged_writer.writerow(interaction)

	fin_merge.close()
	fout_merged.close()

if __name__ == '__main__':
	merge_results()


